@@ -1,4 +1,4 @@
-# Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2014, 2020, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -32,7 +32,11 @@
 # Detection of abstract methods in pylint is not working correctly
 #pylint: disable=W0223
 
-from . import errors
+import os
+import socket
+import sysconfig
+
+from . import errors, version
 from .catch23 import INT_TYPES
 from .constants import (
     CharacterSet, FieldFlag, ServerFlag, ShutdownType, ClientFlag
@@ -71,12 +75,32 @@ class CMySQLConnection(MySQLConnectionAbstract):
                 "MySQL Connector/Python C Extension not available")
         self._cmysql = None
         self._columns = []
+        self._plugin_dir = os.path.join(
+            os.path.dirname(os.path.abspath(_mysql_connector.__file__)),
+            "mysql", "vendor"
+        )
         self.converter = None
         super(CMySQLConnection, self).__init__(**kwargs)
 
         if kwargs:
             self.connect(**kwargs)
 
+    def _add_default_conn_attrs(self):
+        """Add default connection attributes"""
+        license_chunks = version.LICENSE.split(" ")
+        if license_chunks[0] == "GPLv2":
+            client_license = "GPL-2.0"
+        else:
+            client_license = "Commercial"
+
+        self._conn_attrs.update({
+            "_connector_name": "mysql-connector-python",
+            "_connector_license": client_license,
+            "_connector_version": ".".join(
+                [str(x) for x in version.VERSION[0:3]]),
+            "_source_host": socket.gethostname()
+            })
+
     def _do_handshake(self):
         """Gather information of the MySQL server before authentication"""
         self._handshake = {
@@ -99,6 +123,15 @@ class CMySQLConnection(MySQLConnectionAbstract):
         """Returns the server status attribute of MYSQL structure"""
         return self._cmysql.st_server_status()
 
+    def set_allow_local_infile_in_path(self, path):
+        """set local_infile_in_path
+
+        Set allow_local_infile_in_path.
+        """
+
+        if self._cmysql:
+            self._cmysql.set_load_data_local_infile_option(path)
+
     def set_unicode(self, value=True):
         """Toggle unicode mode
 
@@ -150,8 +183,10 @@ class CMySQLConnection(MySQLConnectionAbstract):
             charset_name=charset_name,
             connection_timeout=(self._connection_timeout or 0),
             use_unicode=self._use_unicode,
-            auth_plugin=self._auth_plugin)
-
+            auth_plugin=self._auth_plugin,
+            plugin_dir=self._plugin_dir)
+        if not self.isset_client_flag(ClientFlag.CONNECT_ARGS):
+            self._conn_attrs = {}
         cnx_kwargs = {
             'host': self._host,
             'user': self._user,
@@ -161,14 +196,33 @@ class CMySQLConnection(MySQLConnectionAbstract):
             'client_flags': self._client_flags,
             'unix_socket': self._unix_socket,
             'compress': self.isset_client_flag(ClientFlag.COMPRESS),
-            'ssl_disabled': True
+            'ssl_disabled': True,
+            "conn_attrs": self._conn_attrs,
+            "local_infile": self._allow_local_infile,
+            "load_data_local_dir": self._allow_local_infile_in_path
         }
 
+        tls_versions = self._ssl.get('tls_versions')
+        if tls_versions is not None:
+            tls_versions.sort(reverse=True)
+            tls_versions = ",".join(tls_versions)
+        if self._ssl.get('tls_ciphersuites') is not None:
+            ssl_ciphersuites = self._ssl.get('tls_ciphersuites')[0]
+            tls_ciphersuites = self._ssl.get('tls_ciphersuites')[1]
+        else:
+            ssl_ciphersuites = None
+            tls_ciphersuites = None
+        if tls_versions is not None and "TLSv1.3" in tls_versions and \
+           not tls_ciphersuites:
+            tls_ciphersuites = "TLS_AES_256_GCM_SHA384"
         if not self._ssl_disabled:
             cnx_kwargs.update({
                 'ssl_ca': self._ssl.get('ca'),
                 'ssl_cert': self._ssl.get('cert'),
                 'ssl_key': self._ssl.get('key'),
+                'ssl_cipher_suites': ssl_ciphersuites,
+                'tls_versions': tls_versions,
+                'tls_cipher_suites': tls_ciphersuites,
                 'ssl_verify_cert': self._ssl.get('verify_cert') or False,
                 'ssl_verify_identity':
                     self._ssl.get('verify_identity') or False,
@@ -180,6 +234,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         except MySQLInterfaceError as exc:
             raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
                                              sqlstate=exc.sqlstate)
+
         self._do_handshake()
 
     def close(self):
@@ -259,9 +314,12 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         return None
 
-    def get_rows(self, count=None, binary=False, columns=None, raw=None):
+    def get_rows(self, count=None, binary=False, columns=None, raw=None,
+                 prep_stmt=None):
         """Get all or a subset of rows returned by the MySQL server"""
-        if not (self._cmysql and self.unread_result):
+        unread_result = prep_stmt.have_result_set if prep_stmt \
+            else self.unread_result
+        if not (self._cmysql and unread_result):
             raise errors.InternalError("No result set available")
 
         if raw is None:
@@ -273,7 +331,8 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         counter = 0
         try:
-            row = self._cmysql.fetch_row()
+            row = prep_stmt.fetch_row() if prep_stmt \
+                else self._cmysql.fetch_row()
             while row:
                 if not self._raw and self.converter:
                     row = list(row)
@@ -286,24 +345,33 @@ class CMySQLConnection(MySQLConnectionAbstract):
                 counter += 1
                 if count and counter == count:
                     break
-                row = self._cmysql.fetch_row()
+                row = prep_stmt.fetch_row() if prep_stmt \
+                        else self._cmysql.fetch_row()
             if not row:
-                _eof = self.fetch_eof_columns()['eof']
-                self.free_result()
+                _eof = self.fetch_eof_columns(prep_stmt)['eof']
+                if prep_stmt:
+                    prep_stmt.free_result()
+                    self._unread_result = False
+                else:
+                    self.free_result()
             else:
                 _eof = None
         except MySQLInterfaceError as exc:
-            self.free_result()
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+            if prep_stmt:
+                prep_stmt.free_result()
+                raise errors.InterfaceError(str(exc))
+            else:
+                self.free_result()
+                raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
+                                                 sqlstate=exc.sqlstate)
 
         return rows, _eof
 
-    def get_row(self, binary=False, columns=None, raw=None):
+    def get_row(self, binary=False, columns=None, raw=None, prep_stmt=None):
         """Get the next rows returned by the MySQL server"""
         try:
             rows, eof = self.get_rows(count=1, binary=binary, columns=columns,
-                                      raw=raw)
+                                      raw=raw, prep_stmt=prep_stmt)
             if rows:
                 return (rows[0], eof)
             return (None, eof)
@@ -342,12 +410,15 @@ class CMySQLConnection(MySQLConnectionAbstract):
             raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
                                              sqlstate=exc.sqlstate)
 
-    def fetch_eof_columns(self):
+    def fetch_eof_columns(self, prep_stmt=None):
         """Fetch EOF and column information"""
-        if not self._cmysql.have_result_set:
+        have_result_set = prep_stmt.have_result_set if prep_stmt \
+            else self._cmysql.have_result_set
+        if not have_result_set:
             raise errors.InterfaceError("No result set")
 
-        fields = self._cmysql.fetch_fields()
+        fields = prep_stmt.fetch_fields() if prep_stmt \
+            else self._cmysql.fetch_fields()
         self._columns = []
         for col in fields:
             self._columns.append((
@@ -382,6 +453,46 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         return None
 
+    def cmd_stmt_prepare(self, statement):
+        """Prepares the SQL statement"""
+        if not self._cmysql:
+            raise errors.OperationalError("MySQL Connection not available")
+
+        try:
+            return self._cmysql.stmt_prepare(statement)
+        except MySQLInterfaceError as err:
+            raise errors.InterfaceError(str(err))
+
+    # pylint: disable=W0221
+    def cmd_stmt_execute(self, prep_stmt, *args):
+        """Executes the prepared statement"""
+        try:
+            prep_stmt.stmt_execute(*args)
+        except MySQLInterfaceError as err:
+            raise errors.InterfaceError(str(err))
+
+        self._columns = []
+        if not prep_stmt.have_result_set:
+            # No result
+            self._unread_result = False
+            return self.fetch_eof_status()
+
+        self._unread_result = True
+        return self.fetch_eof_columns(prep_stmt)
+
+    def cmd_stmt_close(self, prep_stmt):
+        """Closes the prepared statement"""
+        if self._unread_result:
+            raise errors.InternalError("Unread result found")
+        prep_stmt.stmt_close()
+
+    def cmd_stmt_reset(self, prep_stmt):
+        """Resets the prepared statement"""
+        if self._unread_result:
+            raise errors.InternalError("Unread result found")
+        prep_stmt.stmt_reset()
+    # pylint: enable=W0221
+
     def cmd_query(self, query, raw=None, buffered=False, raw_as_string=False):
         """Send a query to the MySQL server"""
         self.handle_unread_result()
@@ -442,7 +553,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         :return: Subclass of CMySQLCursor
         :rtype: CMySQLCursor or subclass
         """
-        self.handle_unread_result()
+        self.handle_unread_result(prepared)
         if not self.is_connected():
             raise errors.OperationalError("MySQL Connection not available.")
         if cursor_class is not None:
@@ -558,6 +669,23 @@ class CMySQLConnection(MySQLConnectionAbstract):
         self._charset_id = charset
         self._post_connection()
 
+    def cmd_reset_connection(self):
+        """Resets the session state without re-authenticating
+
+        Works only for MySQL server 5.7.3 or later.
+        """
+        if self._server_version < (5, 7, 3):
+            raise errors.NotSupportedError("MySQL version 5.7.2 and "
+                                           "earlier does not support "
+                                           "COM_RESET_CONNECTION.")
+        try:
+            self._cmysql.reset_connection()
+        except MySQLInterfaceError as exc:
+            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
+                                             sqlstate=exc.sqlstate)
+
+        self._post_connection()
+
     def cmd_refresh(self, options):
         """Send the Refresh command to the MySQL server"""
         try:
@@ -589,7 +717,6 @@ class CMySQLConnection(MySQLConnectionAbstract):
         except MySQLInterfaceError as exc:
             raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
                                              sqlstate=exc.sqlstate)
-
         self.close()
 
     def cmd_statistics(self):
@@ -609,9 +736,55 @@ class CMySQLConnection(MySQLConnectionAbstract):
             raise ValueError("MySQL PID must be int")
         self.info_query("KILL {0}".format(mysql_pid))
 
-    def handle_unread_result(self):
+    def handle_unread_result(self, prepared=False):
         """Check whether there is an unread result"""
+        unread_result = self._unread_result if prepared is True \
+            else self.unread_result
         if self.can_consume_results:
             self.consume_results()
-        elif self.unread_result:
+        elif unread_result:
             raise errors.InternalError("Unread result found")
+
+    def reset_session(self, user_variables=None, session_variables=None):
+        """Clears the current active session
+
+        This method resets the session state, if the MySQL server is 5.7.3
+        or later active session will be reset without re-authenticating.
+        For other server versions session will be reset by re-authenticating.
+
+        It is possible to provide a sequence of variables and their values to
+        be set after clearing the session. This is possible for both user
+        defined variables and session variables.
+        This method takes two arguments user_variables and session_variables
+        which are dictionaries.
+
+        Raises OperationalError if not connected, InternalError if there are
+        unread results and InterfaceError on errors.
+        """
+        if not self.is_connected():
+            raise errors.OperationalError("MySQL Connection not available.")
+
+        try:
+            self.cmd_reset_connection()
+        except (errors.NotSupportedError, NotImplementedError):
+            if self._compress:
+                raise errors.NotSupportedError(
+                    "Reset session is not supported with compression for "
+                    "MySQL server version 5.7.2 or earlier.")
+            elif self._server_version < (5, 7, 3):
+                raise errors.NotSupportedError(
+                    "Reset session is not supported with MySQL server "
+                    "version 5.7.2 or earlier.")
+            else:
+                self.cmd_change_user(self._user, self._password,
+                                     self._database, self._charset_id)
+
+        if user_variables or session_variables:
+            cur = self.cursor()
+            if user_variables:
+                for key, value in user_variables.items():
+                    cur.execute("SET @`{0}` = %s".format(key), (value,))
+            if session_variables:
+                for key, value in session_variables.items():
+                    cur.execute("SET SESSION `{0}` = %s".format(key), (value,))
+            cur.close()
