@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2020, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Converting MySQL and Python types
 """
@@ -32,6 +37,8 @@ from .constants import FieldType, FieldFlag, CharacterSet
 from .catch23 import PY2, NUMERIC_TYPES, struct_unpack
 from .custom_types import HexLiteral
 
+CONVERT_ERROR = "Could not convert '{value}' to python {pytype}"
+
 
 class MySQLConverterBase(object):
     """Base class for conversion classes
@@ -48,6 +55,7 @@ class MySQLConverterBase(object):
         self.use_unicode = None
         self.set_charset(charset)
         self.set_unicode(use_unicode)
+        self._cache_field_types = {}
 
     def set_charset(self, charset):
         """Set character set"""
@@ -74,11 +82,29 @@ class MySQLConverterBase(object):
 
     def to_python(self, vtype, value):
         """Convert MySQL data type to Python"""
-        return value
 
-    def escape(self, buf):
+        if (value == b'\x00' or value is None) and vtype[1] != FieldType.BIT:
+            # Don't go further when we hit a NULL value
+            return None
+
+        if not self._cache_field_types:
+            self._cache_field_types = {}
+            for name, info in FieldType.desc.items():
+                try:
+                    self._cache_field_types[info[0]] = getattr(
+                        self, '_{0}_to_python'.format(name))
+                except AttributeError:
+                    # We ignore field types which has no method
+                    pass
+
+        try:
+            return self._cache_field_types[vtype[1]](value, vtype)
+        except KeyError:
+            return value
+
+    def escape(self, value):
         """Escape buffer for sending to MySQL"""
-        return buf
+        return value
 
     def quote(self, buf):
         """Quote buffer for sending to MySQL"""
@@ -144,14 +170,11 @@ class MySQLConverter(MySQLConverterBase):
             if PY2:
                 if isinstance(buf, float):
                     return repr(buf)
-                else:
-                    return str(buf)
-            else:
-                return str(buf).encode('ascii')
+                return str(buf)
+            return str(buf).encode('ascii')
         elif isinstance(buf, type(None)):
             return bytearray(b"NULL")
-        else:
-            return bytearray(b"'" + buf + b"'")
+        return bytearray(b"'" + buf + b"'")
 
     def to_mysql(self, value):
         """Convert Python data type to MySQL"""
@@ -162,6 +185,39 @@ class MySQLConverter(MySQLConverterBase):
             raise TypeError("Python '{0}' cannot be converted to a "
                             "MySQL type".format(type_name))
 
+    def to_python(self, vtype, value):
+        """Convert MySQL data type to Python"""
+        if value == 0 and vtype[1] != FieldType.BIT:  # \x00
+            # Don't go further when we hit a NULL value
+            return None
+        if value is None:
+            return None
+
+        if not self._cache_field_types:
+            self._cache_field_types = {}
+            for name, info in FieldType.desc.items():
+                try:
+                    self._cache_field_types[info[0]] = getattr(
+                        self, '_{0}_to_python'.format(name))
+                except AttributeError:
+                    # We ignore field types which has no method
+                    pass
+
+        try:
+            return self._cache_field_types[vtype[1]](value, vtype)
+        except KeyError:
+            # If one type is not defined, we just return the value as str
+            try:
+                return value.decode('utf-8')
+            except UnicodeDecodeError:
+                return value
+        except ValueError as err:
+            raise ValueError("%s (field %s)" % (err, vtype[0]))
+        except TypeError as err:
+            raise TypeError("%s (field %s)" % (err, vtype[0]))
+        except:
+            raise
+
     def _int_to_mysql(self, value):
         """Convert value to int"""
         return int(value)
@@ -199,14 +255,13 @@ class MySQLConverter(MySQLConverterBase):
 
     def _bytearray_to_mysql(self, value):
         """Convert value to bytes"""
-        return str(value)
+        return bytes(value)
 
     def _bool_to_mysql(self, value):
         """Convert value to boolean"""
         if value:
             return 1
-        else:
-            return 0
+        return 0
 
     def _nonetype_to_mysql(self, value):
         """
@@ -228,13 +283,13 @@ class MySQLConverter(MySQLConverterBase):
         Returns a bytes.
         """
         if value.microsecond:
-            fmt = '{0:d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}.{6:06d}'
+            fmt = '{0:04d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}.{6:06d}'
             return fmt.format(
                 value.year, value.month, value.day,
                 value.hour, value.minute, value.second,
                 value.microsecond).encode('ascii')
 
-        fmt = '{0:d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}'
+        fmt = '{0:04d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}'
         return fmt.format(
             value.year, value.month, value.day,
             value.hour, value.minute, value.second).encode('ascii')
@@ -248,8 +303,8 @@ class MySQLConverter(MySQLConverterBase):
 
         Returns a bytes.
         """
-        return '{0:d}-{1:02d}-{2:02d}'.format(value.year, value.month,
-                                              value.day).encode('ascii')
+        return '{0:04d}-{1:02d}-{2:02d}'.format(value.year, value.month,
+                                                value.day).encode('ascii')
 
     def _time_to_mysql(self, value):
         """
@@ -306,8 +361,7 @@ class MySQLConverter(MySQLConverterBase):
 
         if PY2:
             return result
-        else:
-            return result.encode('ascii')
+        return result.encode('ascii')
 
     def _decimal_to_mysql(self, value):
         """
@@ -321,45 +375,6 @@ class MySQLConverter(MySQLConverterBase):
 
         return None
 
-    def to_python(self, flddsc, value):
-        """
-        Converts a given value coming from MySQL to a certain type in Python.
-        The flddsc contains additional information for the field in the
-        table. It's an element from MySQLCursor.description.
-
-        Returns a mixed value.
-        """
-        if value == 0 and flddsc[1] != FieldType.BIT:  # \x00
-            # Don't go further when we hit a NULL value
-            return None
-        if value is None:
-            return None
-
-        if not self._cache_field_types:
-            self._cache_field_types = {}
-            for name, info in FieldType.desc.items():
-                try:
-                    self._cache_field_types[info[0]] = getattr(
-                        self, '_{0}_to_python'.format(name))
-                except AttributeError:
-                    # We ignore field types which has no method
-                    pass
-
-        try:
-            return self._cache_field_types[flddsc[1]](value, flddsc)
-        except KeyError:
-            # If one type is not defined, we just return the value as str
-            try:
-                return value.decode('utf-8')
-            except UnicodeDecodeError:
-                return value
-        except ValueError as err:
-            raise ValueError("%s (field %s)" % (err, flddsc[0]))
-        except TypeError as err:
-            raise TypeError("%s (field %s)" % (err, flddsc[0]))
-        except:
-            raise
-
     def row_to_python(self, row, fields):
         """Convert a MySQL text result row to Python types
 
@@ -449,45 +464,66 @@ class MySQLConverter(MySQLConverterBase):
         return struct_unpack('>Q', int_val)[0]
 
     def _DATE_to_python(self, value, dsc=None):  # pylint: disable=C0103
-        """
+        """Converts TIME column MySQL to a python datetime.datetime type.
+
+        Raises ValueError if the value can not be converted.
+
         Returns DATE column type as datetime.date type.
         """
+        if isinstance(value, datetime.date):
+            return value
         try:
             parts = value.split(b'-')
-            return datetime.date(int(parts[0]), int(parts[1]), int(parts[2]))
-        except ValueError:
-            return None
+            if len(parts) != 3:
+                raise ValueError("invalid datetime format: {} len: {}"
+                                 "".format(parts, len(parts)))
+            try:
+                return datetime.date(int(parts[0]), int(parts[1]), int(parts[2]))
+            except ValueError:
+                return None
+        except (IndexError, ValueError):
+            raise ValueError(
+                "Could not convert {0} to python datetime.timedelta".format(
+                    value))
 
     _NEWDATE_to_python = _DATE_to_python
 
     def _TIME_to_python(self, value, dsc=None):  # pylint: disable=C0103
+        """Converts TIME column value to python datetime.time value type.
+
+        Converts the TIME column MySQL type passed as bytes to a python
+        datetime.datetime type.
+
+        Raises ValueError if the value can not be converted.
+
+        Returns datetime.time type.
         """
-        Returns TIME column type as datetime.time type.
-        """
-        time_val = None
         try:
             (hms, mcs) = value.split(b'.')
             mcs = int(mcs.ljust(6, b'0'))
-        except ValueError:
+        except (TypeError, ValueError):
             hms = value
             mcs = 0
         try:
             (hours, mins, secs) = [int(d) for d in hms.split(b':')]
             if value[0] == 45 or value[0] == '-':  # if PY3 or PY2
                 mins, secs, mcs = -mins, -secs, -mcs
-            time_val = datetime.timedelta(hours=hours, minutes=mins,
-                                          seconds=secs, microseconds=mcs)
-        except ValueError:
-            raise ValueError(
-                "Could not convert {0} to python datetime.timedelta".format(
-                    value))
-        else:
-            return time_val
+            return datetime.timedelta(hours=hours, minutes=mins,
+                                      seconds=secs, microseconds=mcs)
+        except (IndexError, TypeError, ValueError):
+            raise ValueError(CONVERT_ERROR.format(value=value,
+                                                  pytype="datetime.timedelta"))
 
     def _DATETIME_to_python(self, value, dsc=None):  # pylint: disable=C0103
+        """"Converts DATETIME column value to python datetime.time value type.
+
+        Converts the DATETIME column MySQL type passed as bytes to a python
+        datetime.datetime type.
+
+        Returns: datetime.datetime type.
         """
-        Returns DATETIME column type as datetime.datetime type.
-        """
+        if isinstance(value, datetime.datetime):
+            return value
         datetime_val = None
         try:
             (date_, time_) = value.split(b' ')
@@ -499,9 +535,21 @@ class MySQLConverter(MySQLConverterBase):
                 mcs = 0
             dtval = [int(i) for i in date_.split(b'-')] + \
                     [int(i) for i in hms.split(b':')] + [mcs, ]
-            datetime_val = datetime.datetime(*dtval)
-        except ValueError:
-            datetime_val = None
+            if len(dtval) < 6:
+                raise ValueError("invalid datetime format: {} len: {}"
+                                 "".format(dtval, len(dtval)))
+            else:
+                # Note that by default MySQL accepts invalid timestamps
+                # (this is also backward compatibility).
+                # Traditionaly C/py returns None for this well formed but
+                # invalid datetime for python like '0000-00-00 HH:MM:SS'.
+                try:
+                    datetime_val = datetime.datetime(*dtval)
+                except ValueError:
+                    return None
+        except (IndexError, TypeError):
+            raise ValueError(CONVERT_ERROR.format(value=value,
+                                                  pytype="datetime.timedelta"))
 
         return datetime_val
 
@@ -517,7 +565,7 @@ class MySQLConverter(MySQLConverterBase):
         return year
 
     def _SET_to_python(self, value, dsc=None):  # pylint: disable=C0103
-        """Returns SET column typs as set
+        """Returns SET column type as set
 
         Actually, MySQL protocol sees a SET as a string type field. So this
         code isn't called directly, but used by STRING_to_python() method.
@@ -546,7 +594,13 @@ class MySQLConverter(MySQLConverterBase):
             if dsc[7] & FieldFlag.SET:
                 return self._SET_to_python(value, dsc)
             if dsc[7] & FieldFlag.BINARY:
-                return value
+                if self.charset != 'binary' and not isinstance(value, str):
+                    try:
+                        return value.decode(self.charset)
+                    except (LookupError, UnicodeDecodeError):
+                        return value
+                else:
+                    return value
 
         if self.charset == 'binary':
             return value
@@ -556,16 +610,13 @@ class MySQLConverter(MySQLConverterBase):
         return value
 
     _VAR_STRING_to_python = _STRING_to_python
+    _JSON_to_python = _STRING_to_python
 
     def _BLOB_to_python(self, value, dsc=None):  # pylint: disable=C0103
-        """Convert BLOB data type to Python"""
+        """Convert BLOB data type to Python."""
         if dsc is not None:
-            if dsc[7] & FieldFlag.BINARY:
-                if PY2:
-                    return value
-                else:
-                    return bytes(value)
-
+            if dsc[7] & FieldFlag.BLOB and dsc[7] & FieldFlag.BINARY:
+                return bytes(value)
         return self._STRING_to_python(value, dsc)
 
     _LONG_BLOB_to_python = _BLOB_to_python
