@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -38,7 +38,9 @@ from .constants import (
 from . import errors, utils
 from .authentication import get_auth_plugin
 from .catch23 import PY2, struct_unpack
-from .errors import get_exception
+from .errors import DatabaseError, get_exception
+
+PROTOCOL_VERSION = 10
 
 
 class MySQLProtocol(object):
@@ -77,9 +79,9 @@ class MySQLProtocol(object):
         return auth_response
 
     def make_auth(self, handshake, username=None, password=None, database=None,
-                  charset=33, client_flags=0,
+                  charset=45, client_flags=0,
                   max_allowed_packet=1073741824, ssl_enabled=False,
-                  auth_plugin=None):
+                  auth_plugin=None, conn_attrs=None):
         """Make a MySQL Authentication packet"""
 
         try:
@@ -96,8 +98,8 @@ class MySQLProtocol(object):
         except AttributeError:
             # Username is already bytes
             username_bytes = username
-        packet = struct.pack('<IIB{filler}{usrlen}sx'.format(
-            filler='x' * 23, usrlen=len(username_bytes)),
+        packet = struct.pack('<IIH{filler}{usrlen}sx'.format(
+            filler='x' * 22, usrlen=len(username_bytes)),
                              client_flags, max_allowed_packet, charset,
                              username_bytes)
 
@@ -111,15 +113,35 @@ class MySQLProtocol(object):
         if client_flags & ClientFlag.PLUGIN_AUTH:
             packet += auth_plugin.encode('utf8') + b'\x00'
 
+        if (client_flags & ClientFlag.CONNECT_ARGS) and conn_attrs is not None:
+            packet += self.make_conn_attrs(conn_attrs)
+
         return packet
 
-    def make_auth_ssl(self, charset=33, client_flags=0,
+    def make_conn_attrs(self, conn_attrs):
+        """Encode the connection attributes"""
+        for attr_name in conn_attrs:
+            if conn_attrs[attr_name] is None:
+                conn_attrs[attr_name] = ""
+        conn_attrs_len = (
+            sum([len(x) + len(conn_attrs[x]) for x in conn_attrs]) +
+            len(conn_attrs.keys()) + len(conn_attrs.values()))
+
+        conn_attrs_packet = struct.pack('<B', conn_attrs_len)
+        for attr_name in conn_attrs:
+            conn_attrs_packet += struct.pack('<B', len(attr_name))
+            conn_attrs_packet += attr_name.encode('utf8')
+            conn_attrs_packet += struct.pack('<B', len(conn_attrs[attr_name]))
+            conn_attrs_packet += conn_attrs[attr_name].encode('utf8')
+        return conn_attrs_packet
+
+    def make_auth_ssl(self, charset=45, client_flags=0,
                       max_allowed_packet=1073741824):
         """Make a SSL authentication packet"""
         return utils.int4store(client_flags) + \
                utils.int4store(max_allowed_packet) + \
-               utils.int1store(charset) + \
-               b'\x00' * 23
+               utils.int2store(charset) + \
+               b'\x00' * 22
 
     def make_command(self, command, argument=None):
         """Make a MySQL packet containing a command"""
@@ -133,7 +155,7 @@ class MySQLProtocol(object):
         return utils.int4store(statement_id) + utils.int4store(rows)
 
     def make_change_user(self, handshake, username=None, password=None,
-                         database=None, charset=33, client_flags=0,
+                         database=None, charset=45, client_flags=0,
                          ssl_enabled=False, auth_plugin=None):
         """Make a MySQL packet with the Change User command"""
 
@@ -172,6 +194,10 @@ class MySQLProtocol(object):
         """Parse a MySQL Handshake-packet"""
         res = {}
         res['protocol'] = struct_unpack('<xxxxB', packet[0:5])[0]
+        if res["protocol"] != PROTOCOL_VERSION:
+            raise DatabaseError("Protocol mismatch; server version = {}, "
+                                "client version = {}".format(res["protocol"],
+                                                             PROTOCOL_VERSION))
         (packet, res['server_version_original']) = utils.read_string(
             packet[5:], end=b'\x00')
 
@@ -353,16 +379,16 @@ class MySQLProtocol(object):
     def _parse_binary_integer(self, packet, field):
         """Parse an integer from a binary packet"""
         if field[1] == FieldType.TINY:
-            format_ = 'b'
+            format_ = '<b'
             length = 1
         elif field[1] == FieldType.SHORT:
-            format_ = 'h'
+            format_ = '<h'
             length = 2
         elif field[1] in (FieldType.INT24, FieldType.LONG):
-            format_ = 'i'
+            format_ = '<i'
             length = 4
         elif field[1] == FieldType.LONGLONG:
-            format_ = 'q'
+            format_ = '<q'
             length = 8
 
         if field[7] & FieldFlag.UNSIGNED:
@@ -374,10 +400,10 @@ class MySQLProtocol(object):
         """Parse a float/double from a binary packet"""
         if field[1] == FieldType.DOUBLE:
             length = 8
-            format_ = 'd'
+            format_ = '<d'
         else:
             length = 4
-            format_ = 'f'
+            format_ = '<f'
 
         return (packet[length:], struct_unpack(format_, packet[0:length])[0])
 
@@ -387,15 +413,15 @@ class MySQLProtocol(object):
         value = None
         if length == 4:
             value = datetime.date(
-                year=struct_unpack('H', packet[1:3])[0],
+                year=struct_unpack('<H', packet[1:3])[0],
                 month=packet[3],
                 day=packet[4])
         elif length >= 7:
             mcs = 0
             if length == 11:
-                mcs = struct_unpack('I', packet[8:length + 1])[0]
+                mcs = struct_unpack('<I', packet[8:length + 1])[0]
             value = datetime.datetime(
-                year=struct_unpack('H', packet[1:3])[0],
+                year=struct_unpack('<H', packet[1:3])[0],
                 month=packet[3],
                 day=packet[4],
                 hour=packet[5],
@@ -411,8 +437,8 @@ class MySQLProtocol(object):
         data = packet[1:length + 1]
         mcs = 0
         if length > 8:
-            mcs = struct_unpack('I', data[8:])[0]
-        days = struct_unpack('I', data[1:5])[0]
+            mcs = struct_unpack('<I', data[8:])[0]
+        days = struct_unpack('<I', data[1:5])[0]
         if data[0] == 1:
             days *= -1
         tmp = datetime.timedelta(days=days,
@@ -506,31 +532,31 @@ class MySQLProtocol(object):
         flags = 0
         if value < 0:
             if value >= -128:
-                format_ = 'b'
+                format_ = '<b'
                 field_type = FieldType.TINY
             elif value >= -32768:
-                format_ = 'h'
+                format_ = '<h'
                 field_type = FieldType.SHORT
             elif value >= -2147483648:
-                format_ = 'i'
+                format_ = '<i'
                 field_type = FieldType.LONG
             else:
-                format_ = 'q'
+                format_ = '<q'
                 field_type = FieldType.LONGLONG
         else:
             flags = 128
             if value <= 255:
-                format_ = 'B'
+                format_ = '<B'
                 field_type = FieldType.TINY
             elif value <= 65535:
-                format_ = 'H'
+                format_ = '<H'
                 field_type = FieldType.SHORT
             elif value <= 4294967295:
-                format_ = 'I'
+                format_ = '<I'
                 field_type = FieldType.LONG
             else:
                 field_type = FieldType.LONGLONG
-                format_ = 'Q'
+                format_ = '<Q'
         return (struct.pack(format_, value), field_type, flags)
 
     def _prepare_binary_timestamp(self, value):
@@ -682,7 +708,7 @@ class MySQLProtocol(object):
                             charset))) + str(value).encode(charset))
                     field_type = FieldType.DECIMAL
                 elif isinstance(value, float):
-                    values.append(struct.pack('d', value))
+                    values.append(struct.pack('<d', value))
                     field_type = FieldType.DOUBLE
                 elif isinstance(value, (datetime.datetime, datetime.date)):
                     (packed, field_type) = self._prepare_binary_timestamp(
